@@ -124,6 +124,7 @@ struct configuration
   conf_node default_node;
   conf_node *thisnode;
   int mtu;          // the mtu used for outgoing tunnel packets
+  int nfmark;       // the SO_MARK // netfilter mark // fwmark
   double rekey;     // rekey interval
   double keepalive; // keepalive probes interval
   char *ifname;     // the interface name (tap0 ...)
@@ -138,6 +139,7 @@ struct configuration
 
   char *script_if_up;
   char *script_node_up;
+  char *script_node_change;
   char *script_node_down;
   char *pidfilename;
 
